using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Linq;

namespace moyashi {
  public class NostrilFix : MVRScript {
    private NostrilFixer nostrilFixer;

    private class CharacterLoadedMonitor : CustomYieldInstruction {
      private DAZCharacterSelector selector;

      public override bool keepWaiting { 
        get { 
          var isReady = selector.selectedCharacter?.skin != null && selector.selectedCharacter != null;
          return !isReady; 
        }
      }

      public CharacterLoadedMonitor(DAZCharacterSelector selector) {
        this.selector = selector;
      }
    }

    private IEnumerator attachFixer(Atom atom) {
      DAZCharacterSelector selector = atom.GetComponentInChildren<DAZCharacterSelector>();
      if (selector != null) {
        CharacterLoadedMonitor characterLoadedMonitor = new CharacterLoadedMonitor(selector);
        yield return StartCoroutine(characterLoadedMonitor);

        DAZSkinV2 skin = selector.selectedCharacter.skin;

        nostrilFixer = skin.GetComponent<NostrilFixer>();
        if (nostrilFixer == null) {
          nostrilFixer = skin.gameObject.AddComponent<NostrilFixer>();
        }

        nostrilFixer.skin = skin;
      } else {
        SuperController.LogError("Atom did not contain a G2 model");
      }
    }

    public override void Init() {
      var greeting = new JSONStorableString("NostrilFixGreeting", "\nNothing to see here :)\n\nIf you want to turn this plugin off just use the enable toggle on the previous screen, or remove the plugin from the Person.\n\n");
      var textField = CreateTextField(greeting);

      LayoutElement layout = textField.GetComponent<LayoutElement>();
      layout.ignoreLayout = true;
      RectTransform rect = textField.GetComponent<RectTransform>();
      rect.sizeDelta = new Vector2(0.0f, 0.0f);
      rect.SetInsetAndSizeFromParentEdge(RectTransform.Edge.Left, 0.0f, 1080.0f); 
      rect.SetInsetAndSizeFromParentEdge(RectTransform.Edge.Top, 0.0f, 1300.0f);

      OnEnable(); // REEE
    }

    public void OnEnable() {
      if (containingAtom == null) return;
      if (containingAtom.type == "Person") {
        StartCoroutine(attachFixer(containingAtom));
      } else {
        SuperController.LogError("NostrilFix only works when attached to a Person");        
      }
    }

    public void OnDisable() {
      if (nostrilFixer != null) {
        Destroy(nostrilFixer);
        nostrilFixer = null;
      }
    }
  }

  public class NostrilFixer : MonoBehaviour {
    public DAZSkinV2 skin;
    private Shader nostrilsOriginalShader;
    private Shader faceShader;
    private Material nostrilsOriginalMaterial;
    private Material nostrilsMaterial;
    // private Material faceMaterial;

    void Start() {
      nostrilsOriginalMaterial = new Material(Shader.Find("Custom/Subsurface/GlossCullComputeBuff"));

      nostrilsMaterial = skin.GPUmaterials.First((m) => m.name.StartsWith("Nostrils"));
      // SuperController.LogMessage($"found: {nostrilsMaterial} {nostrilsMaterial.shader}");

      if (nostrilsMaterial != null) {
        nostrilsOriginalMaterial.CopyPropertiesFromMaterial(nostrilsMaterial);      
        nostrilsOriginalShader = nostrilsMaterial.shader;
      }

      var faceMaterial = skin.GPUmaterials.First((m) => m.name.StartsWith("Face"));
      // SuperController.LogMessage($"found: {faceMaterial} {faceMaterial.shader}");

      if (faceMaterial != null && nostrilsMaterial != null) {
        nostrilsMaterial.shader = faceMaterial.shader;
        nostrilsMaterial.CopyPropertiesFromMaterial(faceMaterial);
      }
    }

    // maybe dont need to do this here after all?
    // void LateUpdate() {
    //   // SuperController.LogMessage("fixer update");
    //   nostrilsMaterial.shader = faceShader;
    //   nostrilsMaterial.CopyPropertiesFromMaterial(faceMaterial);
    // }

    void OnDestroy() {
      if (nostrilsMaterial != null && nostrilsOriginalShader != null) {
        nostrilsMaterial.shader = nostrilsOriginalShader;
        nostrilsMaterial.CopyPropertiesFromMaterial(nostrilsOriginalMaterial);
      }
    }
  }
}